@@ -4,7 +4,7 @@ lfJSDM <- function(formula, data, inits, priors,
 		   n.omp.threads = 1, verbose = TRUE, n.report = 100, 
 		   n.burn = round(.10 * n.samples), 
 		   n.thin = 1, n.chains = 1, k.fold, k.fold.threads = 1, 
-		   k.fold.seed = 100, ...){
+		   k.fold.seed = 100, k.fold.only = FALSE, ...){
 
   ptm <- proc.time()
 
@@ -157,12 +157,6 @@ lfJSDM <- function(formula, data, inits, priors,
       X.re[, j] <- X.re[, j] + max(X.re[, j - 1]) + 1
     }
   }
-  lambda.psi <- matrix(0, J, n.occ.re)
-  if (p.occ.re > 0) {
-    for (i in 1:n.occ.re) {
-      lambda.psi[which(X.re == (i - 1), arr.ind = TRUE)[, 1], i] <- 1
-    }
-  }
 
   # Priors --------------------------------------------------------------
   if (missing(priors)) {
@@ -483,141 +477,141 @@ lfJSDM <- function(formula, data, inits, priors,
 
   # Fit the model ---------------------------------------------------------
   out.tmp <- list()
-  for (i in 1:n.chains) {
-    # Change initial values if i > 1
-    if ((i > 1) & (!fix.inits)) {
-      beta.comm.inits <- rnorm(p.occ, mu.beta.comm, sqrt(sigma.beta.comm))
-      tau.sq.beta.inits <- runif(p.occ, 0.5, 10)
-      beta.inits <- matrix(rnorm(N * p.occ, beta.comm.inits, 
-            		     sqrt(tau.sq.beta.inits)), N, p.occ)
-      beta.inits <- c(beta.inits)
-      lambda.inits <- matrix(0, N, q)
-      diag(lambda.inits) <- 1
-      lambda.inits[lower.tri(lambda.inits)] <- rnorm(sum(lower.tri(lambda.inits)))
-      lambda.inits <- c(lambda.inits)
+  out <- list()
+  if (!k.fold.only) {
+    for (i in 1:n.chains) {
+      # Change initial values if i > 1
+      if ((i > 1) & (!fix.inits)) {
+        beta.comm.inits <- rnorm(p.occ, mu.beta.comm, sqrt(sigma.beta.comm))
+        tau.sq.beta.inits <- runif(p.occ, 0.5, 10)
+        beta.inits <- matrix(rnorm(N * p.occ, beta.comm.inits, 
+              		     sqrt(tau.sq.beta.inits)), N, p.occ)
+        beta.inits <- c(beta.inits)
+        lambda.inits <- matrix(0, N, q)
+        diag(lambda.inits) <- 1
+        lambda.inits[lower.tri(lambda.inits)] <- rnorm(sum(lower.tri(lambda.inits)))
+        lambda.inits <- c(lambda.inits)
+        if (p.occ.re > 0) {
+          sigma.sq.psi.inits <- runif(p.occ.re, 0.5, 10)
+          beta.star.inits <- rnorm(n.occ.re, sqrt(sigma.sq.psi.inits[beta.star.indx + 1]))
+          beta.star.inits <- rep(beta.star.inits, N)
+        }
+      }
+
+      storage.mode(chain.info) <- "integer"
+      # Run the model in C
+      out.tmp[[i]] <- .Call("lfJSDM", y, X, X.re, consts, n.occ.re.long, beta.inits,
+          		  beta.comm.inits, tau.sq.beta.inits, lambda.inits, 
+          		  sigma.sq.psi.inits, beta.star.inits, 
+          		  beta.star.indx, beta.level.indx, 
+          		  mu.beta.comm, Sigma.beta.comm,
+        	                  tau.sq.beta.a, tau.sq.beta.b, 
+          		  sigma.sq.psi.a, sigma.sq.psi.b, 
+          		  n.samples, n.omp.threads, verbose, n.report, 
+        	                  samples.info, chain.info)
+      chain.info[1] <- chain.info[1] + 1
+    }
+    # Calculate R-Hat ---------------
+    out$rhat <- list()
+    if (n.chains > 1) {
+      # as.vector removes the "Upper CI" when there is only 1 variable. 
+      out$rhat$beta.comm <- as.vector(gelman.diag(mcmc.list(lapply(out.tmp, function(a) 
+      					      mcmc(t(a$beta.comm.samples)))), 
+      			     autoburnin = FALSE)$psrf[, 2])
+      out$rhat$tau.sq.beta <- as.vector(gelman.diag(mcmc.list(lapply(out.tmp, function(a) 
+      					      mcmc(t(a$tau.sq.beta.samples)))), 
+      			     autoburnin = FALSE)$psrf[, 2])
+      out$rhat$beta <- as.vector(gelman.diag(mcmc.list(lapply(out.tmp, function(a) 
+      					         mcmc(t(a$beta.samples)))), 
+      			     autoburnin = FALSE)$psrf[, 2])
+      lambda.mat <- matrix(lambda.inits, N, q)
+      out$rhat$lambda.lower.tri <- as.vector(gelman.diag(mcmc.list(lapply(out.tmp, function(a) 
+          					       mcmc(t(a$lambda.samples[c(lower.tri(lambda.mat)), ])))), 
+          					       autoburnin = FALSE)$psrf[, 2])
       if (p.occ.re > 0) {
-        sigma.sq.psi.inits <- runif(p.occ.re, 0.5, 10)
-        beta.star.inits <- rnorm(n.occ.re, sqrt(sigma.sq.psi.inits[beta.star.indx + 1]))
-        beta.star.inits <- rep(beta.star.inits, N)
+        out$rhat$sigma.sq.psi <- as.vector(gelman.diag(mcmc.list(lapply(out.tmp, function(a) 
+        					      mcmc(t(a$sigma.sq.psi.samples)))), 
+        			     autoburnin = FALSE)$psrf[, 2])
+      }
+    } else {
+      out$rhat$beta.comm <- rep(NA, p.occ)
+      out$rhat$tau.sq.beta <- rep(NA, p.occ)
+      out$rhat$beta <- rep(NA, p.occ * N)
+      if (p.occ.re > 0) {
+        out$rhat$sigma.sq.psi <- rep(NA, p.occ.re)
       }
     }
 
-    storage.mode(chain.info) <- "integer"
-    # Run the model in C
-    out.tmp[[i]] <- .Call("lfJSDM", y, X, X.re, consts, n.occ.re.long, beta.inits,
-			  beta.comm.inits, tau.sq.beta.inits, lambda.inits, 
-			  sigma.sq.psi.inits, beta.star.inits, 
-			  beta.star.indx, beta.level.indx, 
-			  mu.beta.comm, Sigma.beta.comm,
-      	                  tau.sq.beta.a, tau.sq.beta.b, 
-			  sigma.sq.psi.a, sigma.sq.psi.b, 
-			  n.samples, n.omp.threads, verbose, n.report, 
-      	                  samples.info, chain.info)
-    chain.info[1] <- chain.info[1] + 1
-  }
-  # Calculate R-Hat ---------------
-  out <- list()
-  out$rhat <- list()
-  if (n.chains > 1) {
-    # as.vector removes the "Upper CI" when there is only 1 variable. 
-    out$rhat$beta.comm <- as.vector(gelman.diag(mcmc.list(lapply(out.tmp, function(a) 
-    					      mcmc(t(a$beta.comm.samples)))), 
-    			     autoburnin = FALSE)$psrf[, 2])
-    out$rhat$tau.sq.beta <- as.vector(gelman.diag(mcmc.list(lapply(out.tmp, function(a) 
-    					      mcmc(t(a$tau.sq.beta.samples)))), 
-    			     autoburnin = FALSE)$psrf[, 2])
-    out$rhat$beta <- as.vector(gelman.diag(mcmc.list(lapply(out.tmp, function(a) 
-    					         mcmc(t(a$beta.samples)))), 
-    			     autoburnin = FALSE)$psrf[, 2])
-    lambda.mat <- matrix(lambda.inits, N, q)
-    out$rhat$lambda.lower.tri <- as.vector(gelman.diag(mcmc.list(lapply(out.tmp, function(a) 
-						       mcmc(t(a$lambda.samples[c(lower.tri(lambda.mat)), ])))), 
-						       autoburnin = FALSE)$psrf[, 2])
-    if (p.occ.re > 0) {
-      out$rhat$sigma.sq.psi <- as.vector(gelman.diag(mcmc.list(lapply(out.tmp, function(a) 
-      					      mcmc(t(a$sigma.sq.psi.samples)))), 
-      			     autoburnin = FALSE)$psrf[, 2])
+    # Put everything into MCMC objects
+    out$beta.comm.samples <- mcmc(do.call(rbind, lapply(out.tmp, function(a) t(a$beta.comm.samples))))
+    colnames(out$beta.comm.samples) <- x.names
+    out$tau.sq.beta.samples <- mcmc(do.call(rbind, 
+      				lapply(out.tmp, function(a) t(a$tau.sq.beta.samples))))
+    colnames(out$tau.sq.beta.samples) <- x.names
+
+    if (is.null(sp.names)) {
+      sp.names <- paste('sp', 1:N, sep = '')
     }
-  } else {
-    out$rhat$beta.comm <- rep(NA, p.occ)
-    out$rhat$tau.sq.beta <- rep(NA, p.occ)
-    out$rhat$beta <- rep(NA, p.occ * N)
+    coef.names <- paste(rep(x.names, each = N), sp.names, sep = '-')
+    out$beta.samples <- mcmc(do.call(rbind, lapply(out.tmp, function(a) t(a$beta.samples))))
+    colnames(out$beta.samples) <- coef.names
     if (p.occ.re > 0) {
-      out$rhat$sigma.sq.psi <- rep(NA, p.occ.re)
+      out$sigma.sq.psi.samples <- mcmc(
+        do.call(rbind, lapply(out.tmp, function(a) t(a$sigma.sq.psi.samples))))
+      colnames(out$sigma.sq.psi.samples) <- x.re.names
+      out$beta.star.samples <- mcmc(
+        do.call(rbind, lapply(out.tmp, function(a) t(a$beta.star.samples))))
+      tmp.names <- unlist(re.level.names)
+      beta.star.names <- paste(rep(x.re.names, n.occ.re.long), tmp.names, sep = '-')
+      beta.star.names <- paste(beta.star.names, rep(sp.names, each = n.occ.re), sep = '-')
+      colnames(out$beta.star.samples) <- beta.star.names
+      out$re.level.names <- re.level.names
+    }
+    loadings.names <- paste(rep(sp.names, times = n.factors), rep(1:n.factors, each = N), sep = '-')
+    out$lambda.samples <- mcmc(do.call(rbind, lapply(out.tmp, function(a) t(a$lambda.samples))))
+    colnames(out$lambda.samples) <- loadings.names
+
+    # Return things back in the original order. 
+    out$z.samples <- do.call(abind, lapply(out.tmp, function(a) array(a$z.samples, 
+      								dim = c(N, J, n.post.samples))))
+    out$z.samples <- aperm(out$z.samples, c(3, 1, 2))
+    out$psi.samples <- do.call(abind, lapply(out.tmp, function(a) array(a$psi.samples, 
+      								dim = c(N, J, n.post.samples))))
+    out$psi.samples <- aperm(out$psi.samples, c(3, 1, 2))
+    out$like.samples <- do.call(abind, lapply(out.tmp, function(a) array(a$like.samples, 
+      								dim = c(N, J, n.post.samples))))
+    out$like.samples <- aperm(out$like.samples, c(3, 1, 2))
+    out$w.samples <- do.call(abind, lapply(out.tmp, function(a) array(a$w.samples, 
+      								dim = c(q, J, n.post.samples))))
+    out$w.samples <- aperm(out$w.samples, c(3, 1, 2))
+    # Calculate effective sample sizes
+    out$ESS <- list()
+    out$ESS$beta.comm <- effectiveSize(out$beta.comm.samples)
+    out$ESS$tau.sq.beta <- effectiveSize(out$tau.sq.beta.samples)
+    out$ESS$beta <- effectiveSize(out$beta.samples)
+    out$ESS$lambda <- effectiveSize(out$lambda.samples)
+    if (p.occ.re > 0) {
+      out$ESS$sigma.sq.psi <- effectiveSize(out$sigma.sq.psi.samples)
+    }
+    out$X <- X
+    out$X.re <- X.re
+    out$y <- y.big
+    out$call <- cl
+    out$n.samples <- n.samples
+    out$x.names <- x.names
+    out$sp.names <- sp.names
+    out$q <- q
+    out$n.post <- n.post.samples
+    out$n.thin <- n.thin
+    out$n.burn <- n.burn
+    out$n.chains <- n.chains
+    out$coords <- coords
+    if (p.occ.re > 0) {
+      out$psiRE <- TRUE
+    } else {
+      out$psiRE <- FALSE
     }
   }
-
-  # Put everything into MCMC objects
-  out$beta.comm.samples <- mcmc(do.call(rbind, lapply(out.tmp, function(a) t(a$beta.comm.samples))))
-  colnames(out$beta.comm.samples) <- x.names
-  out$tau.sq.beta.samples <- mcmc(do.call(rbind, 
-    				lapply(out.tmp, function(a) t(a$tau.sq.beta.samples))))
-  colnames(out$tau.sq.beta.samples) <- x.names
-
-  if (is.null(sp.names)) {
-    sp.names <- paste('sp', 1:N, sep = '')
-  }
-  coef.names <- paste(rep(x.names, each = N), sp.names, sep = '-')
-  out$beta.samples <- mcmc(do.call(rbind, lapply(out.tmp, function(a) t(a$beta.samples))))
-  colnames(out$beta.samples) <- coef.names
-  if (p.occ.re > 0) {
-    out$sigma.sq.psi.samples <- mcmc(
-      do.call(rbind, lapply(out.tmp, function(a) t(a$sigma.sq.psi.samples))))
-    colnames(out$sigma.sq.psi.samples) <- x.re.names
-    out$beta.star.samples <- mcmc(
-      do.call(rbind, lapply(out.tmp, function(a) t(a$beta.star.samples))))
-    tmp.names <- unlist(re.level.names)
-    beta.star.names <- paste(rep(x.re.names, n.occ.re.long), tmp.names, sep = '-')
-    beta.star.names <- paste(beta.star.names, rep(sp.names, each = n.occ.re), sep = '-')
-    colnames(out$beta.star.samples) <- beta.star.names
-    out$re.level.names <- re.level.names
-  }
-  loadings.names <- paste(rep(sp.names, times = n.factors), rep(1:n.factors, each = N), sep = '-')
-  out$lambda.samples <- mcmc(do.call(rbind, lapply(out.tmp, function(a) t(a$lambda.samples))))
-  colnames(out$lambda.samples) <- loadings.names
-
-  # Return things back in the original order. 
-  out$z.samples <- do.call(abind, lapply(out.tmp, function(a) array(a$z.samples, 
-    								dim = c(N, J, n.post.samples))))
-  out$z.samples <- aperm(out$z.samples, c(3, 1, 2))
-  out$psi.samples <- do.call(abind, lapply(out.tmp, function(a) array(a$psi.samples, 
-    								dim = c(N, J, n.post.samples))))
-  out$psi.samples <- aperm(out$psi.samples, c(3, 1, 2))
-  out$like.samples <- do.call(abind, lapply(out.tmp, function(a) array(a$like.samples, 
-    								dim = c(N, J, n.post.samples))))
-  out$like.samples <- aperm(out$like.samples, c(3, 1, 2))
-  out$w.samples <- do.call(abind, lapply(out.tmp, function(a) array(a$w.samples, 
-    								dim = c(q, J, n.post.samples))))
-  out$w.samples <- aperm(out$w.samples, c(3, 1, 2))
-  # Calculate effective sample sizes
-  out$ESS <- list()
-  out$ESS$beta.comm <- effectiveSize(out$beta.comm.samples)
-  out$ESS$tau.sq.beta <- effectiveSize(out$tau.sq.beta.samples)
-  out$ESS$beta <- effectiveSize(out$beta.samples)
-  out$ESS$lambda <- effectiveSize(out$lambda.samples)
-  if (p.occ.re > 0) {
-    out$ESS$sigma.sq.psi <- effectiveSize(out$sigma.sq.psi.samples)
-  }
-  out$X <- X
-  out$X.re <- X.re
-  out$lambda.psi <- lambda.psi
-  out$y <- y.big
-  out$call <- cl
-  out$n.samples <- n.samples
-  out$x.names <- x.names
-  out$sp.names <- sp.names
-  out$q <- q
-  out$n.post <- n.post.samples
-  out$n.thin <- n.thin
-  out$n.burn <- n.burn
-  out$n.chains <- n.chains
-  out$coords <- coords
-  if (p.occ.re > 0) {
-    out$psiRE <- TRUE
-  } else {
-    out$psiRE <- FALSE
-  }
-
   # K-fold cross-validation -------
   if (!missing(k.fold)) {
     if (verbose) {      
@@ -652,8 +646,6 @@ lfJSDM <- function(formula, data, inits, priors,
       J.fit <- nrow(X.fit)
       J.0 <- nrow(X.0)
       # Random occurrence effects
-      lambda.psi.fit <- lambda.psi[-curr.set, , drop = FALSE]
-      lambda.psi.0 <- lambda.psi[curr.set, , drop = FALSE]
       X.re.fit <- X.re[-curr.set, , drop = FALSE]
       X.re.0 <- X.re[curr.set, , drop = FALSE]
       n.occ.re.fit <- length(unique(c(X.re.fit)))
@@ -745,7 +737,7 @@ lfJSDM <- function(formula, data, inits, priors,
       class(out.fit) <- "lfJSDM"
 
       # Predict occurrence at new sites. 
-      if (p.occ.re > 0) {X.0 <- cbind(X.0, X.re.0)}
+      if (p.occ.re > 0) {X.0 <- cbind(X.0, X.re.0 + 1)}
       out.pred <- predict.lfJSDM(out.fit, X.0, coords.0, ignore.RE = FALSE)
       like.samples <- matrix(NA, N, J.0)
       for (q in 1:N) {
