@@ -22,10 +22,16 @@ static const R_CallMethodDef CallEntries[] = {
     {"sfMsPGOccNNGP", (DL_FUNC) &sfMsPGOccNNGP, 61},
     {"sfMsPGOccNNGPPredict", (DL_FUNC) &sfMsPGOccNNGPPredict, 20},
     {"lfJSDM", (DL_FUNC) &lfJSDM, 25},
-    {"sfJSDMNNGP", (DL_FUNC) &sfJSDMNNGP, 42},
+    {"sfJSDMNNGP", (DL_FUNC) &sfJSDMNNGP, 44},
     {"tPGOcc", (DL_FUNC) &tPGOcc, 46},
     {"stPGOccNNGP", (DL_FUNC) &stPGOccNNGP, 64},
     {"stPGOccNNGPPredict", (DL_FUNC) &stPGOccNNGPPredict, 19},
+    {"svcPGBinomNNGP", (DL_FUNC) &svcPGBinomNNGP, 45},
+    {"svcPGOccNNGPPredict", (DL_FUNC) &svcPGOccNNGPPredict, 20},
+    {"svcPGOccNNGP", (DL_FUNC) &svcPGOccNNGP, 59},
+    {"svcTPGBinomNNGP", (DL_FUNC) &svcTPGBinomNNGP, 51},
+    {"svcTPGOccNNGPPredict", (DL_FUNC) &svcTPGOccNNGPPredict, 22},
+    {"svcTPGOccNNGP", (DL_FUNC) &svcTPGBinomNNGP, 65},
     {NULL, NULL, 0}
 };
 
